@@ -1,37 +1,24 @@
+# Copyright © Michal Čihař <michal@weblate.org>
 #
-# Copyright © 2012–2022 Michal Čihař <michal@cihar.com>
-#
-# This file is part of Weblate Client <https://github.com/WeblateOrg/wlc>
-#
-# This program is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <https://www.gnu.org/licenses/>.
-#
+# SPDX-License-Identifier: GPL-3.0-or-later
+
 """Weblate API client library."""
 
+from __future__ import annotations
 import json
 import logging
 from copy import copy
-from typing import Any, Dict, Optional, Set, Tuple
+from typing import Any, ClassVar, Collection, cast
 from urllib.parse import urlencode, urlparse
 
-import dateutil.parser  # type: ignore
+import dateutil.parser
 import requests
 from requests.adapters import HTTPAdapter
 from requests.packages.urllib3.util.retry import Retry
 
 log = logging.getLogger("wlc")
 
-__version__ = "1.13"
+__version__ = "1.14"
 
 URL = "https://weblate.org/"
 DEVEL_URL = "https://github.com/WeblateOrg/wlc"
@@ -44,13 +31,25 @@ TIMESTAMPS = {"last_change"}
 class WeblateException(Exception):
     """Generic error."""
 
-    def __init__(self, message: Optional[str] = None):
+    def __init__(self, message: str | None = None):
         super().__init__(message or self.__doc__)
 
 
 class WeblateThrottlingError(WeblateException):
     """Throttling on the server."""
 
+    def __init__(self, limit: str, retry_after: str):
+        self.limit = limit
+        self.retry_after = retry_after
+        message_segments = [
+            cast(str, self.__doc__)
+        ]  # workaround for https://github.com/python/mypy/issues/15825
+        if limit:
+            message_segments.append(f"Limit is {limit} requests.")
+        if retry_after:
+            message_segments.append(f"Retry after {retry_after} seconds.")
+        super().__init__(" ".join(message_segments))
+
 
 class WeblatePermissionError(WeblateException):
     """You don't have permission to access this object."""
@@ -65,14 +64,14 @@ class Weblate:
 
     def __init__(
         self,
-        key="",
-        url=API_URL,
+        key: str = "",
+        url: str = API_URL,
         config=None,
-        retries=0,
-        status_forcelist=None,
-        method_whitelist=None,
-        backoff_factor=0,
-        timeout=30,
+        retries: int = 0,
+        status_forcelist: Collection[int] | None = None,
+        method_whitelist: Collection[str] | None = None,
+        backoff_factor: int = 0,
+        timeout: int = 300,
     ):
         """Create the object, storing key, API url and requests retry args."""
         self.session = requests.Session()
@@ -128,24 +127,37 @@ class Weblate:
             status_code = error.response.status_code
 
             if status_code == 429:
-                raise WeblateThrottlingError()
+                headers = error.response.headers
+                raise WeblateThrottlingError(
+                    headers.get("X-RateLimit-Limit"), headers.get("Retry-After")
+                ) from error
             if status_code == 404:
                 raise WeblateException(
                     "Object not found on the server "
                     "(maybe operation is not supported on the server)"
-                )
+                ) from error
             if status_code == 403:
-                raise WeblatePermissionError(self.permission_error_message(error))
+                raise WeblatePermissionError(
+                    self.permission_error_message(error)
+                ) from error
 
             if status_code == 401:
-                raise WeblateDeniedError()
+                raise WeblateDeniedError from error
+
+            if 300 <= status_code < 400:
+                raise WeblateException(
+                    "Server responded with an unexpectedHTTP redirect. "
+                    "Please check your configuration."
+                ) from error
 
             reason = error.response.reason
             try:
                 error_string = str(error.response.json())
             except Exception:
                 error_string = ""
-            raise WeblateException(f"HTTP error {status_code}: {reason} {error_string}")
+            raise WeblateException(
+                f"HTTP error {status_code}: {reason} {error_string}"
+            ) from error
 
     def raw_request(self, method, path, data=None, files=None, params=None):
         """Construct request object and returns raw content."""
@@ -159,8 +171,8 @@ class Weblate:
 
         try:
             return response.json()
-        except ValueError:
-            raise WeblateException("Server returned invalid JSON")
+        except ValueError as error:
+            raise WeblateException("Server returned invalid JSON") from error
 
     def invoke_request(self, method, path, data=None, files=None, params=None):
         """Construct request object."""
@@ -174,6 +186,7 @@ class Weblate:
             "headers": headers,
             "verify": verify_ssl,
             "files": files,
+            "allow_redirects": False,
         }
 
         # Disable insecure warnings for localhost
@@ -195,6 +208,8 @@ class Weblate:
             log.debug(json.dumps([method, path, log_kwargs], indent=True))
             response = self.session.request(method, path, **kwargs)
             response.raise_for_status()
+            if 300 <= response.status_code < 400:
+                raise requests.HTTPError("Server redirected", response=response)
         except requests.exceptions.RequestException as error:
             self.process_error(error)
             raise
@@ -227,7 +242,8 @@ class Weblate:
         return parser(weblate=self, **data)
 
     def get_object(self, path):
-        """Return object based on path.
+        """
+        Return object based on path.
 
         Operates on (project, component or translation objects.
         """
@@ -285,6 +301,10 @@ class Weblate:
         """List languages in the instance."""
         return self.list_factory("languages/", Language)
 
+    def list_categories(self, path="categories/"):
+        """List categories in the instance."""
+        return self.list_factory(path, Category)
+
     def add_source_string(
         self, project, component, msgid, msgstr, source_language=None
     ):
@@ -346,10 +366,10 @@ class Weblate:
 class LazyObject(dict):
     """Object which supports deferred loading."""
 
-    PARAMS: Tuple[str, ...] = ()
-    OPTIONALS: Set[str] = set()
-    MAPPINGS: Dict[str, Any] = {}
-    ID = "url"
+    PARAMS: ClassVar[tuple[str, ...]] = ()
+    OPTIONALS: ClassVar[set[str]] = set()
+    MAPPINGS: ClassVar[dict[str, Any]] = {}
+    ID: ClassVar[str] = "url"
 
     def __init__(self, weblate, url, **kwargs):
         """Construct object for given Weblate instance."""
@@ -410,8 +430,8 @@ class LazyObject(dict):
             self.refresh()
         try:
             return self._data[name]
-        except KeyError:
-            raise AttributeError(name)
+        except KeyError as error:
+            raise AttributeError(name) from error
 
     def setattrvalue(self, name, value):
         if name not in self.PARAMS:
@@ -448,14 +468,14 @@ class LazyObject(dict):
 class Language(LazyObject):
     """Language object."""
 
-    PARAMS: Tuple[str, ...] = ("url", "web_url", "code", "name", "direction")
-    ID = "code"
+    PARAMS: ClassVar[tuple[str, ...]] = ("url", "web_url", "code", "name", "direction")
+    ID: ClassVar[str] = "code"
 
 
 class LanguageStats(LazyObject):
     """Language object."""
 
-    PARAMS: Tuple[str, ...] = (
+    PARAMS: ClassVar[tuple[str, ...]] = (
         "total",
         "code",
         "translated_words",
@@ -465,7 +485,7 @@ class LanguageStats(LazyObject):
         "total_words",
         "words_percent",
     )
-    ID = "code"
+    ID: ClassVar[str] = "code"
 
 
 class RepoMixin:
@@ -499,7 +519,12 @@ class RepoMixin:
 class ProjectRepository(LazyObject, RepoMixin):
     """Repository object."""
 
-    PARAMS: Tuple[str, ...] = ("url", "needs_commit", "needs_merge", "needs_push")
+    PARAMS: ClassVar[tuple[str, ...]] = (
+        "url",
+        "needs_commit",
+        "needs_merge",
+        "needs_push",
+    )
 
     def _get_repo_url(self):
         """Return repository url."""
@@ -509,7 +534,7 @@ class ProjectRepository(LazyObject, RepoMixin):
 class Repository(ProjectRepository):
     """Repository object."""
 
-    PARAMS: Tuple[str, ...] = (
+    PARAMS: ClassVar[tuple[str, ...]] = (
         "url",
         "needs_commit",
         "needs_merge",
@@ -534,7 +559,7 @@ class RepoObjectMixin(RepoMixin):
 class Project(LazyObject, RepoObjectMixin):
     """Project object."""
 
-    PARAMS: Tuple[str, ...] = (
+    PARAMS: ClassVar[tuple[str, ...]] = (
         "url",
         "web_url",
         "name",
@@ -542,9 +567,9 @@ class Project(LazyObject, RepoObjectMixin):
         "web",
         "source_language",
     )
-    OPTIONALS = {"source_language"}
-    ID = "slug"
-    MAPPINGS = {"source_language": Language}
+    OPTIONALS: ClassVar[set[str]] = {"source_language"}
+    ID: ClassVar[str] = "slug"
+    MAPPINGS: ClassVar[dict[str, Any]] = {"source_language": Language}
 
     def list(self):
         """List components in the project."""
@@ -561,15 +586,18 @@ class Project(LazyObject, RepoObjectMixin):
         """Return language statistics for component."""
         self.ensure_loaded("languages_url")
         url = self._attribs["languages_url"]
-        return [
-            LanguageStats(self.weblate, url, **item) for item in self.weblate.get(url)
-        ]
+        return [LanguageStats(self.weblate, **item) for item in self.weblate.get(url)]
 
     def changes(self):
         """List changes in the project."""
         self.ensure_loaded("changes_list_url")
         return self.weblate.list_changes(self._attribs["changes_list_url"])
 
+    def categories(self):
+        """List categories in the project."""
+        self.ensure_loaded("categories_url")
+        return self.weblate.list_categories(self._attribs["categories_url"])
+
     def delete(self):
         self.weblate.raw_request("delete", self._url)
 
@@ -577,10 +605,17 @@ class Project(LazyObject, RepoObjectMixin):
         return self.weblate.create_component(self.slug, **kwargs)
 
 
+class Category(LazyObject):
+    """Category object."""
+
+    PARAMS: ClassVar[tuple[str, ...]] = ("category", "name", "project", "slug", "url")
+    MAPPINGS: ClassVar[dict[str, Any]] = {"project": Project}
+
+
 class Component(LazyObject, RepoObjectMixin):
     """Component object."""
 
-    PARAMS: Tuple[str, ...] = (
+    PARAMS: ClassVar[tuple[str, ...]] = (
         "url",
         "web_url",
         "name",
@@ -597,12 +632,25 @@ class Component(LazyObject, RepoObjectMixin):
         "file_format",
         "license",
         "license_url",
+        "agreement",
+        "source_language",
+        "priority",
+        "is_glossary",
+        "category",
+        "linked_component",
+    )
+    OPTIONALS: ClassVar[set[str]] = {
         "source_language",
         "is_glossary",
-    )
-    OPTIONALS = {"source_language", "is_glossary"}
-    ID = "slug"
-    MAPPINGS = {"project": Project, "source_language": Language}
+        "category",
+        "linked_component",
+    }
+    ID: ClassVar[str] = "slug"
+    MAPPINGS: ClassVar[dict[str, Any]] = {
+        "category": Category,
+        "project": Project,
+        "source_language": Language,
+    }
     REPOSITORY_CLASS = Repository
 
     def list(self):
@@ -666,11 +714,14 @@ class Component(LazyObject, RepoObjectMixin):
             url = "{}?{}".format(url, urlencode({"format": convert}))
         return self.weblate.raw_request("get", url)
 
+    def patch(self, **kwargs):
+        return self.weblate.raw_request("patch", self._url, data=kwargs)
+
 
 class Translation(LazyObject, RepoObjectMixin):
     """Translation object."""
 
-    PARAMS: Tuple[str, ...] = (
+    PARAMS: ClassVar[tuple[str, ...]] = (
         "url",
         "web_url",
         "language",
@@ -697,8 +748,8 @@ class Translation(LazyObject, RepoObjectMixin):
         "last_change",
         "last_author",
     )
-    ID = "language_code"
-    MAPPINGS = {"language": Language, "component": Component}
+    ID: ClassVar[str] = "language_code"
+    MAPPINGS: ClassVar[dict[str, Any]] = {"language": Language, "component": Component}
     REPOSITORY_CLASS = Repository
 
     def list(self):
@@ -725,16 +776,13 @@ class Translation(LazyObject, RepoObjectMixin):
             url = "{}?{}".format(url, urlencode({"format": convert}))
         return self.weblate.raw_request("get", url)
 
-    def upload(self, file, overwrite=None, format=None, **kwargs):
+    def upload(self, file, overwrite=None, format=None, **kwargs):  # noqa: A002
         """Updoad a translation file to server."""
         self.ensure_loaded("file_url")
         url = self._attribs["file_url"]
-        if format:
-            files = {"file": (f"file.{format}", file)}
-        else:
-            files = {"file": file}
+        files = {"file": (f"file.{format}", file)} if format else {"file": file}
         if overwrite:
-            kwargs["overwrite"] = "yes"
+            kwargs["conflicts"] = "replace-translated"
 
         return self.weblate.request("post", url, files=files, data=kwargs)
 
@@ -750,7 +798,7 @@ class Translation(LazyObject, RepoObjectMixin):
 class Statistics(LazyObject):
     """Statistics object."""
 
-    PARAMS: Tuple[str, ...] = (
+    PARAMS: ClassVar[tuple[str, ...]] = (
         "failing_percent",
         "translated_percent",
         "total_words",
@@ -770,13 +818,13 @@ class Statistics(LazyObject):
 class TranslationStatistics(Statistics):
     """Translation statistics."""
 
-    PARAMS: Tuple[str, ...] = Statistics.PARAMS + ("code", "last_author")
+    PARAMS: ClassVar[tuple[str, ...]] = (*Statistics.PARAMS, "code", "last_author")
 
 
 class Change(LazyObject):
     """Change object."""
 
-    PARAMS: Tuple[str, ...] = (
+    PARAMS: ClassVar[tuple[str, ...]] = (
         "url",
         "unit",
         "translation",
@@ -785,14 +833,17 @@ class Change(LazyObject):
         "action_name",
         "target",
     )
-    ID = "id"
-    MAPPINGS = {"translation": Translation, "component": Component}
+    ID: ClassVar[str] = "id"
+    MAPPINGS: ClassVar[dict[str, Any]] = {
+        "translation": Translation,
+        "component": Component,
+    }
 
 
 class Unit(LazyObject):
     """Unit object."""
 
-    PARAMS: Tuple[str, ...] = (
+    PARAMS: ClassVar[tuple[str, ...]] = (
         "approved",
         "content_hash",
         "context",
@@ -820,8 +871,8 @@ class Unit(LazyObject):
         "url",
         "web_url",
     )
-    ID = "id"
-    MAPPINGS = {"translation": Translation}
+    ID: ClassVar[str] = "id"
+    MAPPINGS: ClassVar[dict[str, Any]] = {"translation": Translation}
 
     def list(self):
         """API compatibility method, returns self."""
