@@ -1,23 +1,9 @@
-# PYTHON_ARGCOMPLETE_OK
-#
-# Copyright © 2012–2022 Michal Čihař <michal@cihar.com>
-#
-# This file is part of Weblate Client <https://github.com/WeblateOrg/wlc>
-#
-# This program is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <https://www.gnu.org/licenses/>.
+# Copyright © Michal Čihař <michal@weblate.org>
 #
+# SPDX-License-Identifier: GPL-3.0-or-later
+
 """Command-line interface for Weblate."""
+from __future__ import annotations
 import csv
 import http.client
 import json
@@ -26,7 +12,7 @@ import sys
 from argparse import ArgumentParser
 from datetime import datetime
 from pathlib import Path
-from typing import Callable, Dict, List
+from typing import Callable
 
 import argcomplete
 from requests.exceptions import RequestException
@@ -34,9 +20,9 @@ from requests.exceptions import RequestException
 import wlc
 from wlc.config import NoOptionError, WeblateConfig
 
-COMMANDS: Dict[str, Callable] = {}
+COMMANDS: dict[str, Callable] = {}
 
-SORT_ORDER: List[str] = []
+SORT_ORDER: list[str] = []
 
 
 def register_command(command):
@@ -196,9 +182,7 @@ class Command:
                 self.println("    <tr>")
                 for key in header:
                     self.println(
-                        "      <td>{}</td>".format(
-                            self.format_value(getattr(item, key))
-                        )
+                        f"      <td>{self.format_value(getattr(item, key))}</td>"
                     )
                 self.println("    </tr>")
             self.println("  </tbody>")
@@ -241,7 +225,7 @@ class Command:
 
     def run(self):
         """Main execution of the command."""
-        raise NotImplementedError()
+        raise NotImplementedError
 
 
 class ObjectCommand(Command):
@@ -280,13 +264,13 @@ class ObjectCommand(Command):
 
     def run(self):
         """Main execution of the command."""
-        raise NotImplementedError()
+        raise NotImplementedError
 
     @staticmethod
     def check_result(result, message):
         """Check result json data."""
         if not result["result"]:
-            raise CommandError(message, result["detail"] if "detail" in result else "")
+            raise CommandError(message, result.get("detail", ""))
 
 
 class ProjectCommand(ObjectCommand):
@@ -301,7 +285,7 @@ class ProjectCommand(ObjectCommand):
 
     def run(self):
         """Main execution of the command."""
-        raise NotImplementedError()
+        raise NotImplementedError
 
 
 class ComponentCommand(ObjectCommand):
@@ -316,7 +300,7 @@ class ComponentCommand(ObjectCommand):
 
     def run(self):
         """Main execution of the command."""
-        raise NotImplementedError()
+        raise NotImplementedError
 
 
 class TranslationCommand(ObjectCommand):
@@ -331,7 +315,7 @@ class TranslationCommand(ObjectCommand):
 
     def run(self):
         """Main execution of the command."""
-        raise NotImplementedError()
+        raise NotImplementedError
 
 
 @register_command
@@ -552,7 +536,7 @@ class Repo(ObjectCommand):
 
     name = "repo"
     description = (
-        "Displays status of Weblate repository " "for translation, component or project"
+        "Displays status of Weblate repository for translation, component or project"
     )
 
     def run(self):
@@ -566,7 +550,7 @@ class Changes(ObjectCommand):
     """Display repository status for object."""
 
     name = "changes"
-    description = "Displays list of changes " "for translation, component or project"
+    description = "Displays list of changes for translation, component or project"
 
     def run(self):
         """Executor."""
@@ -579,7 +563,7 @@ class Stats(ObjectCommand):
     """Display repository statistics for object."""
 
     name = "stats"
-    description = "Displays statistics " "for translation, component or project"
+    description = "Displays statistics for translation, component or project"
 
     def run(self):
         """Executor."""
@@ -788,7 +772,7 @@ class Upload(TranslationCommand):
         ):
             raise CommandError(
                 "Failed to upload translations!",
-                result["detail"] if "detail" in result else "",
+                result.get("detail", ""),
             )
 
 
@@ -829,7 +813,6 @@ def main(settings=None, stdout=None, stdin=None, args=None):
     command = COMMANDS[args.command](args, config, stdout, stdin)
     try:
         command.run()
-        return 0
     except wlc.WeblateDeniedError:
         url, key = config.get_url_key()
         if key:
@@ -849,6 +832,8 @@ def main(settings=None, stdout=None, stdin=None, args=None):
     except (CommandError, wlc.WeblateException) as error:
         print(f"Error: {error}", file=sys.stderr)
         return 1
+    else:
+        return 0
     finally:
         if args.debug:
             http.client.HTTPConnection.debuglevel = 0
