@@ -1,22 +1,9 @@
+# Copyright © Michal Čihař <michal@weblate.org>
 #
-# Copyright © 2012–2022 Michal Čihař <michal@cihar.com>
-#
-# This file is part of Weblate Client <https://github.com/WeblateOrg/wlc>
-#
-# This program is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <https://www.gnu.org/licenses/>.
-#
+# SPDX-License-Identifier: GPL-3.0-or-later
+
 """Test command-line interface."""
+from __future__ import annotations
 
 import json
 import os
@@ -132,13 +119,13 @@ class TestSettings(APITest):
         config = WeblateConfig()
         self.assertEqual(config.get("weblate", "key"), "")
         self.assertEqual(config.get("weblate", "retries"), 0)
-        self.assertEqual(config.get("weblate", "timeout"), 30)
+        self.assertEqual(config.get("weblate", "timeout"), 300)
         self.assertEqual(
             config.get("weblate", "method_whitelist"),
             "HEAD\nTRACE\nDELETE\nOPTIONS\nPUT\nGET",
         )
         self.assertEqual(config.get("weblate", "backoff_factor"), 0)
-        self.assertEqual(config.get("weblate", "status_forcelist"), None)
+        self.assertIsNone(config.get("weblate", "status_forcelist"))
 
     def test_parsing(self):
         """Test config file parsing."""
@@ -327,7 +314,7 @@ class TestCommands(APITest):
 
     def test_push(self):
         """Project push."""
-        msg = "Error: Failed to push changes!\n" "Push is disabled for Hello/Weblate.\n"
+        msg = "Error: Failed to push changes!\nPush is disabled for Hello/Weblate.\n"
         output = execute(["push", "hello"], expected=1)
         self.assertEqual(msg, output)
 
