@@ -1,30 +1,18 @@
+# Copyright © Michal Čihař <michal@weblate.org>
 #
-# Copyright © 2012–2022 Michal Čihař <michal@cihar.com>
-#
-# This file is part of Weblate Client <https://github.com/WeblateOrg/wlc>
-#
-# This program is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <https://www.gnu.org/licenses/>.
-#
+# SPDX-License-Identifier: GPL-3.0-or-later
+
 """Test the module."""
+from __future__ import annotations
 import io
 import os
-from typing import Any, Optional
+from typing import Any, ClassVar
 
 from requests.exceptions import RequestException
 
 from wlc import (
     API_URL,
+    Category,
     Change,
     Component,
     Project,
@@ -64,7 +52,10 @@ class WeblateErrorTest(APITest):
 
     def test_throttled(self):
         """Test listing projects."""
-        with self.assertRaisesRegex(WeblateException, "Throttling"):
+        with self.assertRaisesRegex(
+            WeblateException,
+            "Throttling.*Limit is 100 requests. Retry after 81818 seconds.",
+        ):
             Weblate().get_object("throttled")
 
     def test_error(self):
@@ -123,6 +114,10 @@ class WeblateTest(APITest):
         """Test listing translations."""
         self.assertEqual(len(list(Weblate().list_translations())), 50)
 
+    def test_categories(self):
+        """Test listing translations."""
+        self.assertEqual(len(list(Weblate().list_categories())), 2)
+
     def test_authentication(self):
         """Test authentication against server."""
         with self.assertRaisesRegex(WeblateException, "permission"):
@@ -236,7 +231,6 @@ class WeblateTest(APITest):
             )
 
     def test_create_component_local_files(self):
-
         test_file = os.path.join(
             os.path.dirname(__file__), "test_data", "mock", "project-local-file.pot"
         )
@@ -296,12 +290,12 @@ class WeblateTest(APITest):
 class ObjectTestBaseClass(APITest):
     """Base class for objects testing."""
 
-    _name: Optional[str] = None
+    _name: str | None = None
     _cls: Any = None
 
     def check_object(self, obj):
         """Perform verification whether object is valid."""
-        raise NotImplementedError()
+        raise NotImplementedError
 
     def get(self):
         """Return remote object."""
@@ -315,7 +309,7 @@ class ObjectTestBaseClass(APITest):
 
     def check_list(self, obj):
         """Perform verification whether listing is valid."""
-        raise NotImplementedError()
+        raise NotImplementedError
 
     def test_list(self):
         """Item listing test."""
@@ -416,6 +410,10 @@ class ProjectTest(ObjectTest):
         stats = obj.statistics()
         self.assertEqual(stats["name"], "Hello")
 
+    def test_categories(self):
+        obj = self.get()
+        self.assertEqual(2, len(list(obj.categories())))
+
     def test_create_component(self):
         """Component creation test."""
         obj = self.get()
@@ -454,6 +452,8 @@ class ComponentTest(ObjectTest):
     def check_object(self, obj):
         """Perform verification whether object is valid."""
         self.assertEqual(obj.name, "Weblate")
+        self.assertEqual(obj.priority, 100)
+        self.assertEqual(obj.agreement, "")
 
     def check_list(self, obj):
         """Perform verification whether listing is valid."""
@@ -493,29 +493,81 @@ class ComponentTest(ObjectTest):
     def test_keys(self):
         """Test keys lazy loading."""
         obj = Component(Weblate(), f"components/{self._name}/")
-        self.assertEqual(
-            sorted(obj.keys()),
-            sorted(
-                [
-                    "branch",
-                    "file_format",
-                    "filemask",
-                    "git_export",
-                    "license",
-                    "license_url",
-                    "name",
-                    "new_base",
-                    "project",
-                    "repo",
-                    "slug",
-                    "source_language",
-                    "source_language",
-                    "template",
-                    "url",
-                    "vcs",
-                    "web_url",
-                ]
-            ),
+        self.assertCountEqual(
+            obj.keys(),
+            [
+                "agreement",
+                "branch",
+                "category",
+                "file_format",
+                "filemask",
+                "git_export",
+                "is_glossary",
+                "license",
+                "license_url",
+                "name",
+                "new_base",
+                "priority",
+                "project",
+                "repo",
+                "slug",
+                "source_language",
+                "source_language",
+                "template",
+                "url",
+                "vcs",
+                "web_url",
+            ],
+        )
+
+    def test_components_patch(self):
+        obj = self.get()
+        resp = obj.patch(priority=80)
+        self.assertIn("--patched--", resp.decode())
+
+
+class ComponentCompatibilityTest(ObjectTest):
+    """Tests a component with lack of all optional fields in a response."""
+
+    _name = "hello/olderweblate"
+    _cls = Component
+
+    def check_object(self, obj):
+        """Perform verification whether object is valid."""
+        self.assertEqual(obj.name, "Weblate")
+        self.assertEqual(obj.priority, 100)
+        self.assertEqual(obj.agreement, "")
+
+    def check_list(self, obj):
+        """Perform verification whether listing is valid."""
+        lst = list(obj)
+        self.assertEqual(len(lst), 33)
+        self.assertIsInstance(lst[0], Translation)
+
+    def test_keys(self):
+        """Test keys lazy loading."""
+        obj = Component(Weblate(), f"components/{self._name}/")
+        self.assertCountEqual(
+            obj.keys(),
+            [
+                "agreement",
+                "branch",
+                "file_format",
+                "filemask",
+                "git_export",
+                "license",
+                "license_url",
+                "name",
+                "new_base",
+                "priority",
+                "project",
+                "repo",
+                "slug",
+                "template",
+                "url",
+                "vcs",
+                "web_url",
+            ],
         )
 
 
@@ -590,7 +642,7 @@ class TranslationTest(ObjectTest):
 class UnitTest(ObjectTestBaseClass):
     _name = "123"
     _cls = Unit
-    patch_data = {
+    patch_data: ClassVar[dict[str, Any]] = {
         "target": ["foo"],
         "state": 30,
     }
@@ -619,6 +671,15 @@ class UnitTest(ObjectTestBaseClass):
         self.assertIn("--deleted--", resp.decode())
 
 
+class CategoryTest(APITest):
+    def test(self):
+        obj = Category(Weblate(), "http://127.0.0.1:8000/api/categories/1/")
+        self.assertIsInstance(obj, Category)
+        self.assertIsNone(obj.category)
+        self.assertEqual(obj.name, "Hi")
+        self.assertEqual(obj.slug, "hi")
+
+
 # Delete the reference, so that the abstract class is not discovered
 # when running tests
 del ObjectTest
